class Adapter {
	
	FolderSelectionToXliffInput(T, Folder, Selection := 0)
	{
		Input.XliffFolder[T] := Folder
		SelectionMode := Config.XliffSelectionMode
		if SelectionMode in FileNameScheme,Folder
		{
			Loop, Files, % Folder "\*.*"
			{
				if (A_LoopFileExt = "xlf" or A_LoopFileExt = "xliff") {
					Input.XliffFile[T] := A_LoopFileName
				}
			}
		} else if (SelectionMode = "Files") {
			for fileName in Selection {
				Input.XliffFile[T] := fileName
			}
		}
		Input.XliffCollection(T)
	}
	
	IniXliffSelection[X]
	{
		set {
			if value {
				SelectionMode := Config.XliffSelectionMode
				if SelectionMode in Files,FileNameScheme
				{
					if (SelectionMode = "Files") {
						Selection := {}
					}
					Loop, parse, value, |
					{
						if (Folder and SelectionMode = "Files") {
							Selection[A_LoopField] := 1
							continue
						} else if Folder {
							Selection := A_LoopField
							break
						}
						Folder := A_LoopField
					}
				} else {
					Folder := value
				}
				if not FileExist(Folder) = "D" {
					Folder := 0
					MsgBox, 8208, % Txt.Get("LoadSettingsFolderNotFoundMsgBoxTitle")
						, % Txt.Get("LoadSettingsFolderNotFoundMsgBoxText", {Folder: Folder})
					return
				} else if not FileExist(Folder "\*.xlf") and not FileExist(Folder "\*.xliff") {
					Folder := 0
					MsgBox, 8208, % Txt.Get("LoadSettingsNoXliffFilesMsgBoxTitle")
						, % Txt.Get("LoadSettingsNoXliffFilesMsgBoxText", {Folder: Folder})
					return
				}
				this.FolderSelectionToXliffInput(X, Folder, Selection)
				if Selection and not IsObject(Selection) {
					Input.XliffSelection[X] := Selection
				}
				this.SetXliffControlsByXliff(X)
				if not (SelectionMode = "FileNameScheme") {
					this.SetControlsByXliff(X)
				}
			}
		}
	}
	
	LinkFromXliff(T)
	{
		Input.LinkXliff(T)
		T2 := (T = "Xt") ? "X2v" : "Xt"
		this.SetXliffControlsByXliff(T2, 1)
		this.SetControlsByXliff(T2, 1)
	}
	
	UnlinkXliff()
	{
		Input.UnlinkXliff()
		Config.XliffLinked := false
	}
	
	SetControlsByXliff(T, ByLink := 0)
	{
		X := (Config.XliffLinked) ? "XL" : T
		ConsiderResult := Input[X].Consider
		SameTargetLangFiles := (ConsiderResult = "E")
			? Input[X].HasProblem["SameTargetLangFiles"]
			: false
		if (X = "Xt" or X = "XL" and ConsiderResult)
		{
			XGui.XtReset()
			XGui.XtConsistencyText := Input[X].ConsistencyText
			XGui.XtConsistency := 1
			TargetLangList := Input[X].TargetLanguageList
			SourceLang := Input[X].SourceLanguage
			if (TargetLangList and not SameTargetLangFiles) {
				XGui.XtSource := 1
				XGui.XtExtractSourceDropDown := TargetLangList
				XGui.XtRefreshSourceFromTargetDropDown := TargetLangList
				XGui.XtExchangeSourceTargetDropDown := TargetLangList
			} else {
				XGui.XtSource := 0
				XGui.XtExtractSourceDropDown := ""
				XGui.XtRefreshSourceFromTargetDropDown := ""
				XGui.XtExchangeSourceTargetDropDown := ""
			}
			if (TargetLangList and SourceLang and not SameTargetLangFiles) {
				XGui.XtRefreshSource := 1
				XGui.XtExchangeSource := 1
			} else {
				XGui.XtRefreshSource := 0
				XGui.XtExchangeSource := 0
			}
			XGui.XtTargetLang := (SourceLang) ? 1 : 0
			XGui.XtSplitMerge := (ConsiderResult) ? 1 : 0
		}
		if (X = "X2v" or X = "XL") {
			XGui.X2vReset()
			if ConsiderResult {
				SourceLang := Input[X].SourceLanguage
				XGui.X2vSourceLangDataSetEdit := SourceLang
				XGui.X2vFromSourceLangText := SourceLang
				XGui.X2vListView := Input[X].List
				XGui.X2vSettings := 1
				XGui.X2vButton := 1
			} else {
				XGui.X2vSourceLangDataSetEdit := ""
				XGui.X2vFromSourceLangText := ""
				XGui.X2vListView := ""
				XGui.X2vSettings := 0
				XGui.X2vButton := 0
				MsgBox, 8208, % Txt.Get("SetX2vControlsByXliffErrorMsgBoxTitle")
					, % Txt.Get("SetX2vControlsByXliffErrorMsgBoxText") 
			}
		}
		if (X = "XL") {
			if ByLink {
				T1 := (T = "Xt") ? "X2v" : "Xt"
			} else {
				T1 := T
			}
			T2 := (T1 = "Xt") ? "X2v" : "Xt"
			XGui[T2 "XliffDropDown", "Choose"] := XGui[T1 "XliffDropDown"]
		}
	}
	
	SetXliffControlsByXliff(T, RecurseT2 := 0)
	{
		if (T = "XL" and not RecurseT2) {
			T := (XGui.ActiveTab = 1) ? "Xt" : "X2v"
		}
		XGui[T "FolderEdit"] := Input.XliffFolder[T]
		XGui[T "XliffDropDown"] := Input.XliffDropDownList[T]
		XGui[T "Xliff"] := 1
		if RecurseT2 {
			return
		}
		XGui[T "XliffDropDown", "Choose"] := "|" 1
		if Config.XliffLinked {
			T2 := (T = "Xt") ? "X2v" : "Xt"
			this.SetXliffControlsByXliff(T2, 1)
		}
	}
	
	SetControlsByAiVariables(ByIni := 0)
	{
		XGui.V2xReset()
		XGui.V2xFileEdit := Input.V2x.File
		Replacements := Input.V2x.InitList()
		XGui.V2xSourceLangDataSetDropDown := Input.V2x.DataSetNameKeys
		XGui.V2xSourceLangDataSetDropDown["Choose"] := "|" 1
		XGui.V2xSettings := 1
		XGui.V2xButton := 1
		if Replacements {
			XGui.V2xChooseTargetLangRadio := 1
			XGui.V2xAllTargetLangRadio["Disable"] := 1
			XGui.V2xListView["CheckAll"] := 1
			if not ByIni {
				MsgBox, 8256, % Txt.Get("V2xListReplaceMsgBoxTitle")
					, % Txt.Get("V2xListReplaceMsgBoxText")
			}
		}
	}
}