class AiVariablesInput {

	Folder := ""
	, FileName := ""
	, FileNameNoExt := ""
	, Xml := ""
	, VDataSetNameKeys := {}
	, VList := ""
	
	File[]
	{
		get {
			return this.Folder "\" this.FileName
		}
		set {
			SplitPath, % value
				, PathFileName
				, PathFolder
				,, PathFileNameNoExt
			this.Folder := PathFolder
			this.FileName := PathFileName
			this.FileNameNoExt := PathFileNameNoExt
			this.Xml := ComObjCreate("MSXML2.DOMDocument.6.0")
			this.Xml.setProperty("ProhibitDTD", false)
			this.Xml.validateOnParse := false
			this.Xml.async := false
			this.Xml.load(value)
			this.Xml.setProperty("SelectionNamespaces", "xmlns:v='http://ns.adobe.com/Variables/1.0/' "
				. "xmlns:s='http://www.w3.org/2000/svg'")
			DataSetNodes := this.Xml.selectNodes("/svg/v:variableSets/v:variableSet/v:sampleDataSets/v:sampleDataSet/@dataSetName "
				. "| /s:svg/v:variableSets/v:variableSet/v:sampleDataSets/v:sampleDataSet/@dataSetName")
			for node in DataSetNodes {
				this.VDataSetNameKeys[node.text] := 1
			}
		}
	}
	
	DataSetNameKeys[Exclude := ""]
	{
		get {
			if Exclude {
				DataSetKeys := this.VDataSetNameKeys.Clone()
				DataSetKeys.Delete(Exclude)
				return DataSetKeys
			} else {
				return this.VDataSetNameKeys
			}
		}
	}
	
	InitList()
	{
		this.VList := {}
		for dataSetName in this.VDataSetNameKeys {
			CleanDataSetName := RegExReplace(dataSetName, "[^a-zA-Z0-9_-]", "", ReplaceCount)
			this.VList[dataSetName] := {}
			if ReplaceCount {
				this.VList[dataSetName].TargetLanguage := CleanDataSetName
				Replacements := 1
			}
		}
		if Replacements {
			return "R"
		}
	}
	
	List[Mode := "Object", ExcludeKey := ""]
	{
		get {
			List := this.VList.Clone()
			if ExcludeKey {
				List.Delete(ExcludeKey)
			}
			if (Mode = "ParameterString") {
				;* XSLT parameter as String e.g. "de|frfrench|en"
				for key in List {
					if List[key].HasKey("Check") {
						if (MoreThanOne) {
							String .= "|"
						}
						String .= (List[key].HasKey("TargetLanguage"))
							? List[key].TargetLanguage "" key
							: key
						MoreThanOne := 1
					}
				}
				return String
			} else if (Mode = "IniString") {
				;* Save in Ini as String e.g. "1de|1frfrench|0en"
				for key in List {
					if MoreThanOne {
						String .= "|"
					}
					String .= (List[key].HasKey("Check")) ? "1" : "0"
					String .= (List[key].HasKey("TargetLanguage"))
						? List[key].TargetLanguage "" key
						: key
					MoreThanOne := 1
				}
				return String
			} else if (Mode = "SelectionLangKeys") {
				;* for processing split output
				SelectionLangKeys := {}
				for dataSet in List {
					if List[dataSet].HasKey("Check") {
						Key := (List[dataSet].HasKey("TargetLanguage"))
							? List[dataSet].TargetLanguage
							: dataSet
						SelectionLangKeys[Key] := 1
					}
				}
				return SelectionLangKeys
			} else {
				return List
			}
		}
		set {
			if (Mode = "Merge") {
				for key, item in value {
					if this.VList.HasKey(key) {
						this.VList[key] := item
					}
				}
			} else if (Mode = "MergeIniString") {
				;* Load from Ini as String e.g. "1de|1frfrench|0en"
				Loop, Parse, value, |, %A_Space%%A_Tab%
				{
					TargetLanguageReplaceCount := 0
					KeyReplaceCount := 0
					Checked := SubStr(A_LoopField, 1, 1)
					KeyString := SubStr(A_LoopField, 3)
					if (CharFoundPos := InStr(KeyString, "")) {
						Key :=  SubStr(KeyString, CharFoundPos + 1)
						TargetLanguage := SubStr(KeyString, 1, CharFoundPos - 1)
						TargetLanguage := RegExReplace(TargetLanguage, "[^a-zA-Z0-9_-]", "", TargetLanguageReplaceCount)
					} else {
						Key := KeyString
						RKey := RegExReplace(Key, "[^a-zA-Z0-9_-]", "", KeyReplaceCount)
					}
					if this.VList.HasKey(Key) {
						this.VList[Key] := {}
						if Checked {
							this.VList[Key, "Check"] := 1
						}
						if CharFoundPos or KeyReplaceCount {
							this.VList[Key, "TargetLanguage"] := (KeyReplaceCount) ? RKey : TargetLanguage
						}
					}
					if TargetLanguageReplaceCount or KeyReplaceCount {
						Replacements := 1
					}
				}
				if Replacements {
					return "R"
				}
			}
		}
	}
}