class Config
{
	static Settings := {Language: "en"
		, ConfigFolder: A_ScriptDir "\config"
		, ConsistencyReportsFolder: A_ScriptDir "\consistency-reports"
		, XsltFolder: A_ScriptDir "\xslt"
		, SemanticUiCssFolder: A_ScriptDir "\Semantic-UI-CSS-master"
		, XliffLinked: 0
		, XliffSelectionMode: "FileNameScheme"
		, Ini: {X2vSourceDataSetName: ""
			, X2vChooseDataSetsFromList: 0
			, V2xSourceLang: {}
			, V2xChooseTargetLangsFromList: 0
			, V2xList: ""}}
	
	Init()
	{
		if (FileExist("xliffer.ini")) {
			this.GetXlifferIniLanguage()
			if (ConfigFolder := this.GetXlifferIniAbsolutePath("XfConfigFolder")) {
				this.Settings.ConfigFolder := ConfigFolder
			}
			if (ConsistencyReportsFolder := this.GetXlifferIniAbsolutePath("XfConsistencyReportsFolder")) {
				this.Settings.ConsistencyReportsFolder := ConsistencyReportsFolder
			}
			if (XsltFolder := this.GetXlifferIniAbsolutePath("XfXsltFolder")) {
				this.Settings.XsltFolder := XsltFolder
			}
			if (SemanticUiCssFolder := this.GetXlifferIniAbsolutePath("XfSemanticUiCssFolder")) {
				this.Settings.SemanticUiCssFolder := SemanticUiCssFolder
			}
		}
	}
	
	GetXlifferIniLanguage()
	{
		IniRead, LangID, xliffer.ini, XlifferLanguage, LanguageID
		if (LangID and LangID != "ERROR") {
			this.Settings.Language := LangID
		}
	}
	
	Language[]
	{
		get {
			return this.Settings.Language
		}
	}
	
	GetXlifferIniAbsolutePath(Key)
	{
		IniRead, AbsolutePath, xliffer.ini, XlifferAbsolutePaths, % Key
		if (FileExist(AbsolutePath) = "D") {
			while SubStr(AbsolutePath, 0) = "\" {
				AbsolutePath := SubStr(AbsolutePath, 1, -1)
			}
			return AbsolutePath
		}
	}
	
	Folder[Name := "Config"]
	{
		get {
			if (Name = "Config") {
				return this.Settings.ConfigFolder
			} else if (Name = "ConsistencyReports") {
				return this.Settings.ConsistencyReportsFolder
			} else if (Name = "Xslt") {
				return this.Settings.XsltFolder
			} else if (Name = "SemanticUiCssFolder") {
				return this.Settings.SemanticUiCssFolder
			}
		}
	}
	
	LoadStandardIni()
	{
		if this.HasStandardIni {
			this.LoadIniValues()
			this.SetControlsByIni()
		}
	}
	
	XliffLinked[]
	{
		get {
			return this.Settings.XliffLinked
		}
		set {
			if not value {
				this.Settings.XliffLinked := 0
			} else {
				this.Settings.XliffLinked := 1
			}
		}
	}
	
	XliffSelectionMode[]
	{
		get {
			return this.Settings.XliffSelectionMode
		}
		set {
			if value in FileNameScheme,Folder,Files
				this.Settings.XliffSelectionMode := value
			if (value = "FileNameScheme") {
				XGui.XtXliffLabel := ""
				XGui.XtXliffText := ""
				XGui.X2vXliffLabel := ""
				XGui.X2vXliffText := ""
			} else {
				XGui.XtXliffLabel := Txt.Get("SelectionXliffGroupBoxLabel")
				XGui.XtXliffText := Txt.Get("SelectionXliffText")
				XGui.X2vXliffLabel := Txt.Get("SelectionXliffGroupBoxLabel")
				XGui.X2vXliffText := Txt.Get("SelectionXliffText")
			}
		}
	}
	
	Reset()
	{
		this.Settings.XliffLinked := 0
		, this.Settings.XliffSelectionMode := "FileNameScheme"
		, this.Settings.Ini.X2vSourceDataSetName := ""
		, this.Settings.Ini.X2vChooseDataSetsFromList := 0
		, this.Settings.Ini.V2xSourceLang := {}
		, this.Settings.Ini.V2xChooseTargetLangsFromList := 0
	}
	
	IniDropDownList[]
	{
		get {
			Loop, Files, % this.Folder "\*.ini"
			{
				if MoreThanOne {
					IniList .= "|"
				}
				if (A_LoopFileName != "standard.ini") {
					IniList .= SubStr(A_LoopFileName, 1, -4)
					MoreThanOne := 1
				}
			}
			return IniList
		}
	}
	
	HasStandardIni[]
	{
		get {
			return (FileExist(this.Folder "\standard.ini")) ? true : false
		}
	}
	
	X2vSourceDataSetName[]
	{
		get {
			return this.Settings.Ini.X2vSourceDataSetName
		}
		set {
			this.Settings.Ini.X2vSourceDataSetName := (Input.XliffSourceLanguage["X2v"] != value)
				? value : ""
		}
	}
	
	X2vChooseDataSetsFromList[]
	{
		get {
			return this.Settings.Ini.X2vChooseDataSetsFromList
		}
		set {
			this.Settings.Ini.X2vChooseDataSetsFromList := (value) ? 1 : 0
		}
	}
	
	V2xSourceLang[]
	{
		get {
			for SourceLang, DataSetName in this.Settings.Ini.V2xSourceLang {
				if DataSetName {
					String := DataSetName ""
				}
				String .= SourceLang
				break
			}
			return String
		}
		set {
			if (CharFoundPos := InStr(value, "")) {
				SourceLang := SubStr(value, CharFoundPos + 1)
				DataSetName := SubStr(value, 1, CharFoundPos - 1)
			} else {
				SourceLang := value
			}
			this.Settings.Ini.V2xSourceLang[SourceLang] := DataSetName
		}
	}
	
	V2xChooseTargetLangsFromList[]
	{
		get {
			return this.Settings.Ini.V2xChooseTargetLangsFromList
		}
		set {
			this.Settings.Ini.V2xChooseTargetLangsFromList := (value) ? 1 : 0
		}
	}
	
	V2xList[]
	{
		get {
			return this.Settings.Ini.V2xList
		}
		set {
			this.Settings.Ini.V2xList := value
		}
	}
	
	LoadIni(Ini := "standard.ini")
	{
		if not RegExMatch(Ini, "i)\.ini$") {
			Ini .= ".ini"
		}
		if (FileExist(IniFile := this.Folder "\" Ini)) {
			this.Reset()
			XGui.Reset()
			this.LoadIniValues(Ini)
			this.SetControlsByIni()
			return 1
		}
	}
	
	LoadIniValues(Ini := "standard.ini")
	{
		IniFile := this.Folder "\" Ini
		this.XliffLinked := this.GetIniValue(IniFile, "XlifferOptions", "XliffLinked")
		this.XliffSelectionMode := this.GetIniValue(IniFile, "XlifferOptions", "XliffSelectionMode")
		if this.XliffLinked and LinkedXliffSelection := this.GetIniValue(IniFile, "XliffTools", "LinkedXliffSelection") {
			Adapter.IniXliffSelection["XL"] := LinkedXliffSelection
		} else {
			if (XtXliffSelection := this.GetIniValue(IniFile, "XliffTools", "XtXliffSelection")) {
				Adapter.IniXliffSelection["Xt"] := XtXliffSelection
			}
			if (X2vXliffSelection := this.GetIniValue(IniFile, "XliffToAiVariables", "X2vXliffSelection")) {
				Adapter.IniXliffSelection["X2v"] := X2vXliffSelection
			}
		}
		this.X2vSourceDataSetName := this.GetIniValue(IniFile, "XliffToAiVariables", "X2vSourceDataSetName")
		this.X2vChooseDataSetsFromList := this.GetIniValue(IniFile, "XliffToAiVariables", "X2vChooseDataSetsFromList")
		X := (this.XliffLinked) ? "XL" : "X2v"
		if IsObject(Input[X]) and IsObject(Input[X].List) {
			Input[X].List["MergeIniString"] := this.GetIniValue(IniFile, "XliffToAiVariables", "X2vList")
		}
		if (AiVariablesInputFile := this.GetIniValue(IniFile, "XliffFromAiVariables", "AiVariablesInputFile")) {
			Input.AiVariablesFile := AiVariablesInputFile	
		}
		this.V2xSourceLang := this.GetIniValue(IniFile, "XliffFromAiVariables", "V2xSourceLang")
		this.V2xChooseTargetLangsFromList := this.GetIniValue(IniFile, "XliffFromAiVariables", "V2xChooseTargetLangsFromList")
		this.V2xList := this.GetIniValue(IniFile, "XliffFromAiVariables", "V2xList")
	}
	
	SetControlsByIni()
	{
		CurrentTab := XGui.ActiveTab
		XGui.ActiveTab := "X2v"
		X := "X2v"
		Input.Linked(X)
		if IsObject(Input[X]) {
			if this.X2vSourceDataSetName {
				XGui.X2vSourceLangDataSetEdit := this.X2vSourceDataSetName
			}
			if this.X2vChooseDataSetsFromList {
				XGui.X2vChooseTargetLangRadio := 1
			} else {
				XGui.X2vAllTargetLangRadio := 1
				if IsObject(Input[X].List) {
					XGui.X2vListView["Checkable"] := 1
				}
			}
			if IsObject(Input[X].List) {
				XGui.X2vListView := Input[X].List
			}
		}
		
		XGui.ActiveTab := "V2x"
		if IsObject(Input.V2x) {
			Adapter.SetControlsByAiVariables(1)
			for SourceLang, DataSetName in this.Settings.Ini.V2xSourceLang {
				V2xSourceLang := SourceLang
				XGui.V2xSourceLangDataSetDropDown["Choose"] := SourceLang
				XGui.V2xSourceLangEdit := (DataSetName) ? DataSetName : SourceLang
				break
			}
			if this.V2xChooseTargetLangsFromList or not XGui.V2xAllTargetLangRadio["IsEnabled"] {
				XGui.V2xChooseTargetLangRadio := 1
			} else {
				XGui.V2xAllTargetLangRadio := 1
				if IsObject(Input.V2x.List) {
					XGui.V2xListView["Checkable"] := 1
				}
			}
			if IsObject(Input.V2x.List) {
				Input.V2x.List["MergeIniString"] := this.V2xList
				XGui.V2xListView := Input.V2x.List["Object", V2xSourceLang]
			}
		}

		XGui.ActiveTab := CurrentTab
		
		TrayMenu.XliffLinked := this.XliffLinked
		XliffSelectionMode := this.XliffSelectionMode
		if (XliffSelectionMode = "FileNameScheme") {
			TrayMenu.XliffSelectionFileNameScheme := 1
		} else if (XliffSelectionMode = "Folder") {
			TrayMenu.XliffSelectionXliffFolder := 1
		} else if (XliffSelectionMode = "Files") {
			TrayMenu.XliffSelectionXliffFiles := 1
		}
	}
	
	GetControlsForIni()
	{
		this.X2vSourceDataSetName := XGui.X2vSourceLangDataSetEdit
		this.X2vChooseDataSetsFromList := XGui.X2vChooseTargetLangRadio
		X := "X2v"
		Input.Linked(X)
		if IsObject(Input[X]) and IsObject(Input[X].List) {
			Input[X].List["Merge"] := XGui.X2vListView
		}
		V2xSourceLangChoice := XGui.V2xSourceLangDataSetDropDown
		V2xSourceLangEditText := XGui.V2xSourceLangEdit
		this.Settings.Ini.V2xSourceLang := {}
		this.Settings.Ini.V2xSourceLang[V2xSourceLangChoice] := (V2xSourceLangChoice != V2xSourceLangEditText) 
			? V2xSourceLangEditText : ""
		this.V2xChooseTargetLangsFromList := XGui.V2xChooseTargetLangRadio
		if IsObject(Input.V2x) and IsObject(Input.V2x.List) {
			Input.V2x.List["Merge"] := XGui.V2xListView
		}
	}
	
	SaveIni(Ini := "standard.ini")
	{
		this.GetControlsForIni()
		if not RegExMatch(Ini, "i)\.ini$") {
			Ini .= ".ini"
		}
		IniFile := this.Folder "\" Ini
		Overwrite := FileExist(IniFile)
		this.SetIniValue(this.XliffLinked, IniFile, "XlifferOptions", "XliffLinked")
		this.SetIniValue(this.XliffSelectionMode, IniFile, "XlifferOptions", "XliffSelectionMode")
		if this.XliffLinked {
			this.SetIniValue(Input.XL.IniSelection, IniFile, "XliffTools", "LinkedXliffSelection")
			if Overwrite {
				IniDelete, % IniFile, XliffTools, XtXliffSelection
				IniDelete, % IniFile, XliffToAiVariables, X2vXliffSelection
			}
		} else {
			this.SetIniValue(Input.Xt.IniSelection, IniFile, "XliffTools", "XtXliffSelection")
			this.SetIniValue(Input.X2v.IniSelection, IniFile, "XliffToAiVariables", "X2vXliffSelection")
			if Overwrite {
				IniDelete, % IniFile, XliffTools, LinkedXliffSelection
			}
		}
		this.SetIniValue(this.X2vSourceDataSetName, IniFile, "XliffToAiVariables", "X2vSourceDataSetName")
		this.SetIniValue(this.X2vChooseDataSetsFromList, IniFile, "XliffToAiVariables", "X2vChooseDataSetsFromList")
		X := "X2v"
		Input.Linked(X) 
		this.SetIniValue(Input[X].List["IniString"], IniFile, "XliffToAiVariables", "X2vList")
		V2xInputFile := IsObject(Input.V2x) ? Input.V2x.Folder "\" Input.V2x.FileName : ""
		this.SetIniValue(V2xInputFile, IniFile, "XliffFromAiVariables", "AiVariablesInputFile")
		this.SetIniValue(this.V2xSourceLang, IniFile, "XliffFromAiVariables", "V2xSourceLang")
		this.SetIniValue(this.V2xChooseTargetLangsFromList, IniFile, "XliffFromAiVariables", "V2xChooseTargetLangsFromList")
		this.SetIniValue(Input.V2x.List["IniString"], IniFile, "XliffFromAiVariables", "V2xList")
		return 1
	}
	
	GetIniValue(IniFile, Section, Key)
	{
		IniRead, Value, % IniFile, % Section, % Key
		if (Value != "ERROR") {
			return Value
		}
	}
	
	SetIniValue(Value, IniFile, Section, Key)
	{
		IniWrite, % Value, % IniFile, % Section, % Key
		if ErrorLevel {
			MsgBox , 8208, % Txt.Get("SaveSettingsIniErrorMsgBoxTitle")
				, % Txt.Get("SaveSettingsIniErrorMsgBoxText", {IniFile: IniFile})
		}
	}
}