class SaveXliffDialog extends DialogBox
{
	static Mode
		, Input
		, Hwnd := {}
		, Folder
		, Different
		, DefaultOriginal
		, DefaultDatatype
		, DefaultFile

	Box()
	{
		global SaveXliffFolderEdit
			, SaveXliffFolderButton
			, TargetToSourceRadio
			, OriginalCheckBox
			, OriginalEdit
			, OriginalComboBox
			, DatatypeCheckBox
			, DatatypeEdit
			, DatatypeComboBox
			, SplitRadio
			, MergeRadio
			, FileEditLabel
			, FileEdit
			, LostFocusControl
		static Text1
		this.Mode := Mode := Transformer.Settings.Mode
		X := (Mode = "V2x") ? "V2x" : "Xt"
		X := (X = "Xt" and Config.XliffLinked) ? "XL" : X
		this.Input := X
		this.Folder := Input[X].Folder
		this.Different := {}
		if (X = "V2x") {
			this.DefaultOriginal := Input.V2x.FileName
			this.DefaultDatatype := "html"
			this.DefaultFile := Input.V2x.FileNameNoExt
		} else {
			if IsObject(Original := Input[X].Original) {
				this.Different.Original := 1
				for o in Original {
					if (A_Index > 1) {
						OriginalOptions .= "|"
					}
					OriginalOptions .= o
					OriginalRows := A_Index
				}
				this.DefaultOriginal := OriginalOptions
			} else if Original {
				this.DefaultOriginal := Original
			} else {
				this.DefaultOriginal := Txt.Get("NoValueEditText")
			}
			if IsObject(Datatype := Input[X].Datatype) {
				this.Different.Datatype := 1
				for d in Datatype {
					if (A_Index > 1) {
						DatatypeOptions .= "|"
					}
					DatatypeOptions .= d
					DatatypeRows := A_Index
				}
				this.DefaultDatatype := DatatypeOptions
			} else if Datatype {
				this.DefaultDatatype := Datatype
			} else {
				this.DefaultDatatype := Txt.Get("NoValueEditText")
			}
			FilePrefix := (Mode = "TargetLang") ? Transformer.Settings.Parameter.targetLanguage "." : ""
			if IsObject(BaseName := Input[X].BaseName) {
				this.Different.BaseName := 1
				for b in BaseName {
					if (A_Index > 1) {
						BaseNameOptions .= "|"
					}
					BaseNameOptions .= FilePrefix b
					BaseNameRows := A_Index
				}
				this.DefaultFile := BaseNameOptions
			} else {
				this.DefaultFile := FilePrefix BaseName
			}
		}
		if (Mode = "ExtractSource") {
			BoxTitle := Txt.Get("SaveXliffDialogExtractSourceTitle")
			SplitMerge := 0
			TargetToSource := 1
		} else if (Mode = "TargetLang") {
			BoxTitle := Txt.Get("SaveXliffDialogTargetLangTitle")
			SplitMerge := 0
		} else if (Mode = "Split") {
			BoxTitle := Txt.Get("SaveXliffDialogSplitTitle")
			SplitMerge := 0
		} else if (Mode = "Merge") {
			BoxTitle := Txt.Get("SaveXliffDialogMergeTitle")
			SplitMerge := 0
		} else {
			BoxTitle := Txt.Get("SaveXliffDialogDefaultTitle")
			SplitMerge := 1
		}
		this.InitGui(BoxTitle)
		Gui, Add, Text, % "x" this.MainPaddingLeft " y" this.MainPaddingTopBottom " vText1 Section"
			, % Txt.Get("SaveXliffDialogOutputFolder")
		Gui, Add, Edit, xp0 yp20 w333 h20 ReadOnly vSaveXliffFolderEdit, % this.Folder
		Gui, Add, Button, % "xp336 yp-1 w30 h22 vSaveXliffFolderButton HwndOpenButton g" this.__Class ".FolderButton"
		GuiButtonIcon(OpenButton, "resources\folder-open.ico")
		this.GuiWidth := 350
		Gui, Font, bold
		Gui, Add, Text, xs0 yp38, % Txt.Get("SaveXliffDialogSaveOptions")
		Gui, Font,
		if TargetToSource {
			Gui, Add, Radio,  xs0 yp25 Checked
				, % Txt.Get("SaveXliffDialogSourceToSourceRadio")
			Gui, Add, Radio, vTargetToSourceRadio
				, % Txt.Get("SaveXliffDialogTargetToSourceRadio")
		}
		Gui, Add, CheckBox, % "yp25 vOriginalCheckBox g" this.__Class ".OriginalCheckBox"
			, % Txt.Get("SaveXliffDialogOriginalCheckBox")
		if OriginalOptions {
			Gui, Add, ComboBox, % "xp16 yp18 w347 r" OriginalRows " vOriginalComboBox HwndH", % this.DefaultOriginal
			ControlSetText,, % Txt.Get("DifferentValuesComboBoxText"), ahk_id %H%
			CtlColors.Attach(H, "", "808080")
		} else {
			Gui, Add, Edit, % "xp16 yp18 w347 h20 vOriginalEdit HwndH c808080", % this.DefaultOriginal
		}
		this.Hwnd.Original := H
		Gui, Add, CheckBox, % "xs0 yp33 vDatatypeCheckBox g" this.__Class ".DatatypeCheckBox"
			, % Txt.Get("SaveXliffDialogDatatypeCheckBox")
		if DatatypeOptions {
			Gui, Add, ComboBox, % "xp16 yp18 w347 r" DatatypeRows " vDatatypeComboBox HwndH", % this.DefaultDatatype
			ControlSetText,, % Txt.Get("DifferentValuesComboBoxText"), ahk_id %H%
			CtlColors.Attach(H, "", "808080")
		} else {
			Gui, Add, Edit, % "xp16 yp18 w347 h20 vDatatypeEdit HwndH c808080", % this.DefaultDatatype
		}
		this.Hwnd.Datatype := H
		if SplitMerge {
			Gui, Add, Radio, % "xs0 yp35 vSplitRadio g" this.__Class ".SplitRadio Checked"
				, % Txt.Get("SaveXliffDialogSplitRadio")
			Gui, Add, Radio, % "vMergeRadio g" this.__Class ".MergeRadio"
				, % Txt.Get("SaveXliffDialogMergeRadio")
		}
		Gui, Add, Text, % "xs0 yp" ((SplitMerge) ? 24 : 37) " vFileEditLabel"
			, % (Mode = "Merge" or Mode = "ExtractSource" or Mode = "TargetLang" or Mode = "Merge")
				? Txt.Get("SaveXliffDialogFileEditLabel")
				: Txt.Get("SaveXliffDialogFileSetEditLabel")
		if BaseNameOptions {
			Gui, Add, ComboBox, % "xp0 yp20 w363 r" BaseNameRows " vFileEdit HwndH g" this.__Class ".FileEdit", % this.DefaultFile
			GuiControl, Choose, FileEdit, 1
		} else {
			Gui, Add, Edit, % "xp0 yp20 w363 h20 vFileEdit HwndH g" this.__Class ".FileEdit", % this.DefaultFile
		}
		this.Hwnd.File := H
		GuiControlGet, ControlPos, Pos, FileEdit
		this.ControlsHeight := ControlPosY + ControlPosH
		Buttons := [{Name: Txt.Get("DialogButtonOK"), Action: this.__Class ".OK", Default: 1}
			, {Name: Txt.Get("DialogButtonCancel"), Action: this.__Class ".Cancel"}]
		this.AddButtonBar(Buttons*)
		this.ShowGui()
		OnMessage(0x201, "WM_LBUTTONDOWN")
	}
	
	OriginalCheckBox()
	{
		global OriginalCheckBox, OriginalEdit
		GuiControlGet, CheckState,, OriginalCheckBox
		if CheckState {
			OriginalHwnd := SaveXliffDialog.Hwnd.Original
			ControlClick,, ahk_id %OriginalHwnd%
		} else {
			if SaveXliffDialog.Different.HasKey("Original") {
				CtlColors.Change(SaveXliffDialog.Hwnd.Original, "", "808080")
			} else {
				GuiControl, +c808080, OriginalEdit
				GuiControl, MoveDraw, OriginalEdit
			}
		}
	}
	
	DatatypeCheckBox()
	{
		global DatatypeCheckBox, DatatypeEdit
		GuiControlGet, CheckState,, DatatypeCheckBox
		if CheckState {
			DatatypeHwnd := SaveXliffDialog.Hwnd.Datatype
			ControlClick,, ahk_id %DatatypeHwnd%
		} else {
			if SaveXliffDialog.Different.HasKey("Datatype") {
				CtlColors.Change(SaveXliffDialog.Hwnd.Datatype, "", "808080")
			} else {
				GuiControl, +c808080, DatatypeEdit
				GuiControl, MoveDraw, DatatypeEdit
			}
		}
	}
	
	SplitRadio()
	{
		global FileEditLabel
		GuiControl,, FileEditLabel, % Txt.Get("SaveXliffDialogFileSetEditLabel")
	}
	
	MergeRadio()
	{
		global FileEditLabel
		GuiControl,, FileEditLabel, % Txt.Get("SaveXliffDialogFileEditLabel")
	}
	
	FileEdit()
	{
		global LostFocusControl
		Handle := SaveXliffDialog.Hwnd.File
		LostFocusControl[Handle] := "FileFocusLost"
	}
	
	FolderButton()
	{
		global SaveXliffFolderEdit
		FileSelectFolder, Folder, % "*" SaveXliffDialog.Folder, 1, % Txt.Get("SaveXliffDialogSelectFolderPrompt")
		if Folder {
			GuiControl,, SaveXliffFolderEdit, % Folder
		}
	}
	
	OK()
	{
		global SaveXliffFolderEdit
			, TargetToSourceRadio
			, OriginalCheckBox
			, OriginalComboBox
			, OriginalEdit
			, DatatypeCheckBox
			, DatatypeComboBox
			, DatatypeEdit
			, SplitRadio
			, FileEdit
		Mode := SaveXliffDialog.Mode
		GuiControlGet, Folder,, SaveXliffFolderEdit
		Transformer.Settings.Output.Folder := Folder
		if (Mode = "ExtractSource") {
			GuiControlGet, TargetToSourceChecked,, TargetToSourceRadio
			if TargetToSourceChecked {
				Transformer.Settings.Parameter.targetToSource := true
			}
		}
		GuiControlGet, OriginalChecked,, OriginalCheckBox
		if OriginalChecked {
			if SaveXliffDialog.Different.HasKey("Original") {
				GuiControlGet, Original,, OriginalComboBox
			} else {
				GuiControlGet, Original,, OriginalEdit
			}
			Transformer.Settings.Output.Original := Original
		}
		GuiControlGet, DatatypeChecked,, OriginalCheckBox
		if DatatypeChecked {
			if SaveXliffDialog.Different.HasKey("Datatype") {
				GuiControlGet, Original,, DatatypeComboBox
			} else {
				GuiControlGet, Original,, DatatypeEdit
			}
			Transformer.Settings.Output.Datatype := Datatype
		}
		if Mode not in ExtractSource,TargetLang,Split,Merge
		{
			GuiControlGet, SplitChecked,, SplitRadio
			if SplitChecked {
				Transformer.Settings.Output.Split := 1
			} else {
				Transformer.Settings.Output.Merge := 1
			}
		} else if (Mode = "Split") {
			Transformer.Settings.Output.Split := 1
		} else if (Mode = "Merge") {
			Transformer.Settings.Output.Merge := 1
		}
		GuiControlGet, File,, FileEdit
		if not File {
			MsgBox, 8208, % Txt.Get("SaveXliffDialogNoFileNameMsgBoxTitle")
				, % Txt.Get("SaveXliffDialogNoFileNameMsgBoxText")
			return
		}
		if (ExtFoundPos := RegExMatch(File, "i)\.(xlf|xliff|xml)$")) {
			File := SubStr(File, 1, ExtFoundPos -1)
		}
		File .= ".xlf"
		Transformer.Settings.Output.File := File
		if (Transformer.Transform()) {
			MsgBox, 8256, % Txt.Get("SaveXliffDialogDoneMsgBoxTitle")
				, % Txt.Get("SaveXliffDialogDoneMsgBoxText")
		}
		OnMessage(0x201, "")
		base.CloseGui()
		
	}
	
	Cancel()
	{
		OnMessage(0x201, "")
		base.CloseGui()
	}
}

WM_LBUTTONDOWN(wParam, lParam)
{
	global OriginalEdit
		, OriginalCheckBox
		, OriginalComboBox
		, DatatypeEdit
		, DatatypeCheckBox
		, DatatypeComboBox
		, LostFocusControl
    X := lParam & 0xFFFF
    Y := lParam >> 16
    if (A_GuiControl = "OriginalEdit") {
		GuiControl, Focus, OriginalCheckBox
		GuiControlGet, EditText,, OriginalEdit
		GuiControl,, OriginalCheckBox, 1
		if (EditText = Txt.Get("NoValueEditText")) {
			GuiControl,, OriginalEdit
		}
		GuiControl, -Color, OriginalEdit
		LostFocusControl[SaveXliffDialog.Hwnd.Original] := "OriginalEditFocusLost"
	} else if (A_GuiControl = "OriginalComboBox") {
		GuiControl, Focus, OriginalCheckBox
		GuiControlGet, ComboBoxText,, OriginalComboBox
		GuiControl,, OriginalCheckBox, 1
		Handle := SaveXliffDialog.Hwnd.Original
		if (ComboBoxText = Txt.Get("DifferentValuesComboBoxText")) {
			ControlSetText,,, ahk_id %Handle%
		}
		CtlColors.Change(Handle, "", "")
		LostFocusControl[Handle] := "OriginalComboBoxFocusLost"
	} else if (A_GuiControl = "DatatypeEdit") {
		GuiControl, Focus, DatatypeCheckBox
		GuiControlGet, EditText,, DatatypeEdit
		GuiControl,, DatatypeCheckBox, 1
		if (EditText = Txt.Get("NoValueEditText")) {
			GuiControl,, DatatypeEdit
		}
		GuiControl, -Color, DatatypeEdit
		LostFocusControl[SaveXliffDialog.Hwnd.Datatype] := "DatatypeEditFocusLost"
	} else if (A_GuiControl = "DatatypeComboBox") {
		GuiControl, Focus, DatatypeCheckBox
		GuiControlGet, ComboBoxText,, DatatypeComboBox
		GuiControl,, DatatypeCheckBox, 1
		Handle := SaveXliffDialog.Hwnd.Datatype
		if (EditText = Txt.Get("DifferentValuesComboBoxText")) {
			ControlSetText,,, ahk_id %Handle%
		}
		CtlColors.Change(Handle, "", "")
		LostFocusControl[Handle] := "DatatypeComboBoxFocusLost"
	}
}

OriginalEditFocusLost()
{
	global OriginalCheckBox, OriginalEdit, LostFocusControl
	GuiControlGet, EditText,, OriginalEdit
	CheckText := SaveXliffDialog.DefaultOriginal
	if not EditText or EditText = CheckText {
		GuiControl,, OriginalCheckBox, 0
		GuiControl, +c808080, OriginalEdit
		if not EditText {
			GuiControl,, OriginalEdit, % CheckText
		}
	}
	LostFocusControl.Delete(SaveXliffDialog.Hwnd.Original)
}

OriginalComboBoxFocusLost()
{
	global OriginalCheckBox, OriginalComboBox, LostFocusControl
	GuiControlGet, ComboBoxText,, OriginalComboBox
	CheckText := Txt.Get("DifferentValuesComboBoxText")
	Handle := SaveXliffDialog.Hwnd.Original
	if not ComboBoxText or ComboBoxText = CheckText {
		GuiControl,, OriginalCheckBox, 0
		CtlColors.Change(Handle, "", "808080")
		if not ComboBoxText {
			ControlSetText,, % CheckText, ahk_id %Handle%
		}
	}
	LostFocusControl.Delete(Handle)
}

DatatypeEditFocusLost()
{
	global DatatypeCheckBox, DatatypeEdit, LostFocusControl
	GuiControlGet, EditText,, DatatypeEdit
	CheckText := SaveXliffDialog.DefaultDatatype
	if not EditText or EditText = CheckText {
		GuiControl,, DatatypeCheckBox, 0
		GuiControl, +c808080, DatatypeEdit
		if not EditText {
			GuiControl,, OriginalEdit, % CheckText
		}
	}
	LostFocusControl.Delete(SaveXliffDialog.Hwnd.Datatype)
}

DatatypeComboBoxFocusLost()
{
	global DatatypeCheckBox, DatatypeComboBox, LostFocusControl
	GuiControlGet, ComboBoxText,, DatatypeComboBox
	CheckText := Txt.Get("DifferentValuesComboBoxText")
	Handle := SaveXliffDialog.Hwnd.Datatype
	if not ComboBoxText or ComboBoxText = CheckText {
		GuiControl,, DatatypeCheckBox, 0
		CtlColors.Change(Handle, "", "808080")
		if not ComboBoxText {
			ControlSetText,, % CheckText, ahk_id %Handle%
		}
	}
	LostFocusControl.Delete(Handle)
}

FileEditFocusLost()
{
	global FileEdit, LostFocusControl
	GuiControlGet, EditText,, FileEdit
	if not EditText {
		GuiControl,, FileEdit, % SaveXliffDialog.DefaultFile
	}
	LostFocusControl.Delete(SaveXliffDialog.Hwnd.File)
}