class Transformer {

	static XliffExtractSourceXslt := ComObjCreate("MSXML2.FreeThreadedDOMDocument.6.0")
		, XliffRefreshSourceXslt := ComObjCreate("MSXML2.FreeThreadedDOMDocument.6.0")
		, XliffExchangeSourceXslt := ComObjCreate("MSXML2.FreeThreadedDOMDocument.6.0")
		, XliffNewTargetXslt := ComObjCreate("MSXML2.FreeThreadedDOMDocument.6.0")
		, XliffToAiVariablesXslt := ComObjCreate("MSXML2.FreeThreadedDOMDocument.6.0")
		, XliffOutputXslt := ComObjCreate("MSXML2.FreeThreadedDOMDocument.6.0")
		, XliffFromAiVariablesXslt := ComObjCreate("MSXML2.FreeThreadedDOMDocument.6.0")
		, Settings := {Mode: ""
			, Parameter: {}
			, Output: {}}
	
	Init()
	{
		XsltFolder := Config.Folder["Xslt"]
		
		this.XliffExtractSourceXslt.async := false
		this.XliffExtractSourceXslt.load(XsltFolder "\xliff-extract-source.xsl")
		
		this.XliffRefreshSourceXslt.async := false
		this.XliffRefreshSourceXslt.load(XsltFolder "\xliff-refresh-source.xsl")
		
		this.XliffExchangeSourceXslt.async := false
		this.XliffExchangeSourceXslt.load(XsltFolder "\xliff-exchange-source.xsl")
		
		this.XliffNewTargetXslt.async := false
		this.XliffNewTargetXslt.load(XsltFolder "\xliff-new-target.xsl")
		
		this.XliffToAiVariablesXslt.async := false
		this.XliffToAiVariablesXslt.load(XsltFolder "\xliff-to-ai-variables.xsl")
		
		this.XliffOutputXslt.async := false
		this.XliffOutputXslt.load(XsltFolder "\xliff-output.xsl")
		
		this.XliffFromAiVariablesXslt.async := false
		this.XliffFromAiVariablesXslt.load(XsltFolder "\xliff-from-ai-variables.xsl")
	}
	
	Transform()
	{
		Settings := this.Settings
		Mode := Settings.Mode
		if (Mode = "V2x") {
			X := "V2x"
		} else {
			T := (Mode = "X2v") ? "X2v" : "Xt"
			X := (Config.XliffLinked) ? "XL" : T
		}
		if Mode not in Split,Merge
		{
			Xslt := ComObjCreate("MSXML2.XSLTemplate.6.0")
			if (Mode = "ExtractSource") {
				Xslt.stylesheet := this.XliffExtractSourceXslt
			} else if (Mode = "RefreshSource") {
				Xslt.stylesheet := this.XliffRefreshSourceXslt
			} else if (Mode = "ExchangeSource") {
				Xslt.stylesheet := this.XliffExchangeSourceXslt
			} else if (Mode = "TargetLang") {
				Xslt.stylesheet := this.XliffNewTargetXslt
			} else if (Mode = "X2v") {
				Xslt.stylesheet := this.XliffToAiVariablesXslt
			} else if (Mode = "V2x") {
				Xslt.stylesheet := this.XliffFromAiVariablesXslt
			}
			XsltProc := Xslt.createProcessor()
			XsltProc.input := Input[X].Xml
			for name, value in Settings.Parameter {
				XsltProc.addParameter(name, value)
			}
			XsltProc.transform()
		}
		if Mode in ExtractSource,TargetLang,X2v
		{
			this.WriteXmlResultFile(Settings.Output.Folder, Settings.Output.File, XsltProc.output)
		} else {
			OutXslt := ComObjCreate("MSXML2.XSLTemplate.6.0")
			OutXslt.stylesheet := this.XliffOutputXslt
			if Mode not in Split,Merge
			{
				TempXml := ComObjCreate("MSXML2.DOMDocument.6.0")
				TempXml.async := false
				TempXml.validateOnParse := false
				TempXml.loadXml(XsltProc.output)
			}
			if Settings.Output.HasKey("Split") {
				if Settings.Output.HasKey("SourceLang") {
					SourceOutXsltProc := OutXslt.createProcessor()
					if (Mode = "Split") {
						SourceOutXsltProc.input := Input[X].Xml
					} else {
						SourceOutXsltProc.input := TempXml
					}
					SourceOutXsltProc.addParameter("sourceLanguage", Settings.Output.SourceLang)
					if Settings.Output.HasKey("Original") {
						SourceOutXsltProc.addParameter("original", Settings.Output.Original)
					}
					if Settings.Output.HasKey("Datatype") {
						SourceOutXsltProc.addParameter("datatype", Settings.Output.Datatype)
					}
					SourceOutXsltProc.transform()
					this.WriteXmlResultFile(Settings.Output.Folder, Settings.Output.File, SourceOutXsltProc.output)
				}
				for targetLangCode in Settings.Output.TargetLang {
					TargetOutXsltProc%A_Index% := OutXslt.createProcessor()
					if (Mode = "Split") {
						TargetOutXsltProc%A_Index%.input := Input[X].Xml
					} else {
						TargetOutXsltProc%A_Index%.input := TempXml
					}
					TargetOutXsltProc%A_Index%.addParameter("targetLanguage", targetLangCode)
					if Settings.Output.HasKey("Original") {
						TargetOutXsltProc%A_Index%.addParameter("original", Settings.Output.Original)
					}
					if Settings.Output.HasKey("Datatype") {
						TargetOutXsltProc%A_Index%.addParameter("datatype", Settings.Output.Datatype)
					}
					TargetOutXsltProc%A_Index%.transform()
					this.WriteXmlResultFile(Settings.Output.Folder, targetLangCode "." Settings.Output.File, TargetOutXsltProc%A_Index%.output)
				}
			} else {
				MergeOutXsltProc := OutXslt.createProcessor()
				if (Mode = "Merge") {
					MergeOutXsltProc.input := Input[X].Xml
				} else {
					MergeOutXsltProc.input := TempXml
				}
				if Settings.Output.HasKey("Original") {
					MergeOutXsltProc.addParameter("original", Settings.Output.Original)
				}
				if Settings.Output.HasKey("Datatype") {
					MergeOutXsltProc.addParameter("datatype", Settings.Output.Datatype)
				}
				MergeOutXsltProc.transform()
				this.WriteXmlResultFile(Settings.Output.Folder, Settings.Output.File, MergeOutXsltProc.output)
			}
		}
		return 1
	}
	
	WriteXmlResultFile(ByRef Folder, ByRef File, ByRef Xml)
	{
		if FileExist(Folder "\" File) {
			MsgBox, 8228, % Txt.Get("WriteXmlResultFileOverwriteMsgBoxTitle")
				, % Txt.Get("WriteXmlResultFileOverwriteMsgBoxText", {File: File})
			IfMsgBox No
				return
		}
		XmlResultFile := FileOpen(Folder "\" File, "w", "UTF-8-RAW")
		if !IsObject(XmlResultFile)
		{
			MsgBox, 8208, % Txt.Get("WriteXmlResultFileErrorMsgBoxTitle")
				, % Txt.Get("WriteXmlResultFileErrorMsgBoxText", {File: File})
			return
		}
		XmlResultFile.Write(Xml)
		XmlResultFile.Close()
	}
	
	Reset()
	{
		this.Settings := {Mode: "", Parameter: {}, Output: {}}
	}
}
