class XliffInput {
	
	Folder := ""
	, XFile := {}
	, XSelection := ""
	, XfCollection := ""
	, SourceLanguage := ""
	, TargetLanguageKeys := ""
	, XfDocuments := ""
	, XfConsistency := ""
	, XStatus := {ConsiderResult: ""
		, ConsistencyReport: ""}
	, Xml := ""
	, Original := ""
	, Datatype := ""
	, XList := ""
	
	__New()
	{
		if (Config.XliffSelectionMode = "Files") {
			this.XSelection := {}
		}
	}
	
	Selection[]
	{
		get {
			Selection := {Folder: this.Folder, File: {}}
			;* after this.XliffCollection()
			if IsObject(this.XfDocuments) {
				DocumentNodes := this.XfDocuments.selectNodes("/xf:documents/xf:document")
				for docNode in DocumentNodes {
					FileName := docNode.text
					BaseNameNode := docNode.selectSingleNode("@xf:docBaseName")
					PrefixNode := docNode.selectSingleNode("@xf:docPrefix")
					Selection.File[FileName] := {BaseName: BaseNameNode.text}
					if IsObject(PrefixNode) {
						Selection.File[FileName].Prefix := PrefixNode.text
					}
				}
			;* Files-SelectionMode
			} else if IsObject(this.XSelection) {
				for fileName in this.XSelection {
					for baseName, baseItem in this.XFile {
						if baseItem.HasKey(fileName) {
							Selection.File[fileName] := baseItem[fileName]
							break
						}
					}
				}
			;* FileNameScheme-SelectionMode
			} else if this.XSelection {
				 Selection.File := this.XFile[this.XSelection]
			;* Folder-SelectionMode
			} else {
				for baseName, baseItem in this.XFile {
					for fileName, fileItem in baseItem {
						Selection.File[fileName] := fileItem
					}
				}
			}
			return Selection
		}
		set {
			;* happens only in FileNameScheme-SelectionMode
			this.XSelection := value
			this.XfCollection := ""
			, this.SourceLanguage := ""
			, this.TargetLanguageKeys := ""
			, this.XfDocuments := ""
			, this.XfConsistency := ""
			, this.XStatus := {ConsiderResult: ""
				, ConsistencyReport: ""}
			, this.Xml := ""
			, this.Original := ""
			, this.Datatype := ""
			, this.XList := ""
			this.XliffCollection()
		}
	}
	
	IniSelection[]
	{
		get {
			SelectionString := this.Folder
			;* Files-SelectionMode
			if IsObject(this.XSelection) {
				for fileName in this.XSelection {
					SelectionString .= "|" fileName
				}
			;* FileNameScheme-SelectionMode
			} else if this.XSelection {
				SelectionString .= "|" this.XSelection
			}
			return SelectionString
		}
	}
	
	BaseNameKeys[]
	{
		get {
			BaseNameKeys := {}
			If IsObject(this.XfCollection) {
				BaseNameNodes := this.XfCollection.selectNodes("/xf:collection/xlf:xliff/@xf:docBaseName")
				for node in BaseNameNodes {
					BaseName := node.text
					if not BaseNameKeys.HasKey(BaseName) {
						BaseNameKeys[BaseName] := 1
					}
				}
			} else {
				for baseName in this.XFile {
					BaseNameKeys[baseName] := 1
				}
			}
			return BaseNameKeys
		}
	}
	
	BaseNameList[Seperator := "|"]
	{
		get {
			for baseName in this.BaseNameKeys {
				if (A_Index > 1) {
					BaseNameList .= Seperator
				}
				BaseNameList .= baseName
			}
			return BaseNameList
		}
	}
	
	BaseName[]
	{
		get {
			;* needed in SaveXliffDialog / SaveVarDialog
			BaseNameNodes := this.XfDocuments.selectNodes("/xf:documents/xf:document/@xf:docBaseName")
			BaseNameKeys := {}
			for node in BaseNameNodes {
				if BaseNameKeys.HasKey(node.text) {
					continue
				}
				BaseNameKeys[node.text] := 1
				if (A_Index > 1) {
					MultipleBaseNames := true
				}
			}
			if MultipleBaseNames {
				return BaseNameKeys
			} else {
				for baseName in BaseNameKeys {
					return baseName
				}
			}
		}
	}
	
	File[]
	{
		set {
			SplitPath, % value
				, FileName
				,,, FileNameNoExt
			if (DotFoundPos := InStr(FileNameNoExt, ".")) {
				Prefix := SubStr(FileNameNoExt, 1, DotFoundPos - 1)
				, BaseName := SubStr(FileNameNoExt, DotFoundPos + 1)
			} else {
				BaseName := FileNameNoExt
			}
			if not this.XFile.HasKey(BaseName) {
				this.XFile[BaseName] := {}
			}
			this.XFile[BaseName, FileName] := {}
			this.XFile[BaseName, FileName].BaseName := BaseName
			if Prefix {
				this.XFile[BaseName, FileName].Prefix := Prefix
			}
			;* Files-SelectionMode
			if IsObject(this.XSelection) {
				this.XSelection[FileName] := 1
			}
		}
	}
	
	XliffCollection()
	{
		this.CreateXfCollection()
		this.XfCollection.setProperty("SelectionNamespaces", "xmlns:xf='xl:ffer' xmlns:xlf='urn:oasis:names:tc:xliff:document:1.2'")
		SourceLangNode := this.XfCollection.selectSingleNode("/xf:collection/xlf:xliff/xlf:file[@source-language and not(@target-language)]/@source-language")
		if IsObject(SourceLangNode) {
			this.SourceLanguage := SourceLangNode.text
		}
		TargetLangNodes := this.XfCollection.selectNodes("/xf:collection/xlf:xliff/xlf:file/@target-language")
		if (TargetLangNodes.length) {
			this.TargetLanguageKeys := {}
			for node in TargetLangNodes {
				this.TargetLanguageKeys[node.text] := 1
			}
			this.XList := this.TargetLanguageKeys.Clone()
		}
		this.CreateXfDocuments()
		this.XfDocuments.setProperty("SelectionNamespaces", "xmlns:xf='xl:ffer'")
		this.CreateXfConsistency()
		this.XfConsistency.setProperty("SelectionNamespaces", "xmlns:xf='xl:ffer'")
		this.XliffConsider()
		if (this.XStatus.ConsiderResult) {
			this.CreateXml()
			this.Xml.setProperty("SelectionNamespaces", "xmlns:xlf='urn:oasis:names:tc:xliff:document:1.2'")
			for key, attribute in ["Original", "Datatype"] {
				StringLower, attributeName, attribute
				XPath := "/xlf:xliff/xlf:file/@" attributeName
				if (this.HasProblem["Different" attribute "s"]) {
					this[attribute] := {}
					Nodes := this.Xml.selectNodes(XPath)
					for node in Nodes {
						this[attribute, node.text] := 1
					}
				} else {
					Node := this.Xml.selectSingleNode(XPath)
					if IsObject(Node) {
						this[attribute] := Node.text
					}
				}
			}
		}
	}
	
	CreateXfCollection()
	{
		global SchemaCache
		
		Selection := this.Selection
		Folder := Selection.Folder
		this.XfCollection := ComObjCreate("MSXML2.DOMDocument.6.0")
		this.XfCollection.async := false
		this.XfCollection.validateOnParse := false
		XfCollectionRoot = <?xml version="1.0"?>`n<xf:collection xmlns="urn:oasis:names:tc:xliff:document:1.2" xmlns:xf="xl:ffer"></xf:collection>
		this.XfCollection.loadXml(XfCollectionRoot)
		XfCollectionNode := this.XfCollection.documentElement
		XfFolderAttribute := this.XfCollection.createNode(2, "xf:folder", "xl:ffer")
		XfFolderAttributeValue := this.XfCollection.createTextNode(Folder)
		XfFolderAttribute.appendChild(XfFolderAttributeValue)
		XfCollectionNode.setAttributeNode(XfFolderAttribute)
		for fileName, fileItem in Selection.File {
			if (FileExist(AbsolutePath := Folder "\" fileName)) {
				XmlDoc := ComObjCreate("MSXML2.DOMDocument.6.0")
				XmlDoc.async := false
				XmlDoc.validateOnParse := false
				XmlDoc.load(AbsolutePath)
				if (XmlDoc.parseError.errorCode) {
					MsgBox, 8208, % Txt.Get("XmlWellFormednessErrorMsgBoxTitle")
						, % Txt.Get("XmlWellFormednessErrorMsgBoxText", {File: fileName})
					continue
				}
				XliffNode := XmlDoc.documentElement
				XfDocumentAttribute := XmlDoc.createNode(2, "xf:document", "xl:ffer")
				XfDocumentAttributeValue := XmlDoc.createTextNode(fileName)
				XfDocumentAttribute.appendChild(XfDocumentAttributeValue)
				XliffNode.setAttributeNode(XfDocumentAttribute)
				XfDocBaseNameAttribute := XmlDoc.createNode(2, "xf:docBaseName", "xl:ffer")
				XfDocBaseNameAttributeValue := XmlDoc.createTextNode(fileItem.BaseName)
				XfDocBaseNameAttribute.appendChild(XfDocBaseNameAttributeValue)
				XliffNode.setAttributeNode(XfDocBaseNameAttribute)
				if fileItem.Prefix {
					XfDocPrefixAttribute := XmlDoc.createNode(2, "xf:docPrefix", "xl:ffer")
					XfDocPrefixAttributeValue := XmlDoc.createTextNode(fileItem.Prefix)
					XfDocPrefixAttribute.appendChild(XfDocPrefixAttributeValue)
					XliffNode.setAttributeNode(XfDocPrefixAttribute)
				}
				XmlValDoc := ComObjCreate("MSXML2.DOMDocument.6.0")
				XmlValDoc.async := false
				XmlValDoc.schemas := SchemaCache
				XmlValDoc.validateOnParse := true
				XmlValDoc.resolveExternals := true
				XmlValDoc.load(AbsolutePath)
				if (XmlValDoc.parseError.errorCode) {
					ParseErrorElement := XmlDoc.createNode(1, "xf:parseError", "xl:ffer")
					ErrorCodeElement := XmlDoc.createNode(1, "xf:errorCode", "xl:ffer")
					ErrorCodeElementValue := XmlDoc.createTextNode(XmlValDoc.parseError.errorCode)
					ErrorCodeElement.appendChild(ErrorCodeElementValue)
					ParseErrorElement.appendChild(ErrorCodeElement)
					FileposElement := XmlDoc.createNode(1, "xf:filepos", "xl:ffer")
					FileposElementValue := XmlDoc.createTextNode(XmlValDoc.parseError.filepos)
					FileposElement.appendChild(FileposElementValue)
					ParseErrorElement.appendChild(FileposElement)
					LineElement := XmlDoc.createNode(1, "xf:line", "xl:ffer")
					LineElementValue := XmlDoc.createTextNode(XmlValDoc.parseError.line)
					LineElement.appendChild(LineElementValue)
					ParseErrorElement.appendChild(LineElement)
					LineposElement := XmlDoc.createNode(1, "xf:linepos", "xl:ffer")
					LineposElementValue := XmlDoc.createTextNode(XmlValDoc.parseError.linepos)
					LineposElement.appendChild(LineposElementValue)
					ParseErrorElement.appendChild(LineposElement)
					ReasonElement := XmlDoc.createNode(1, "xf:reason", "xl:ffer")
					ReasonElementValue := XmlDoc.createTextNode(XmlValDoc.parseError.reason)
					ReasonElement.appendChild(ReasonElementValue)
					ParseErrorElement.appendChild(ReasonElement)
					SrcTextElement := XmlDoc.createNode(1, "xf:srcText", "xl:ffer")
					SrcTextElementValue := XmlDoc.createTextNode(XmlValDoc.parseError.srcText)
					SrcTextElement.appendChild(SrcTextElementValue)
					ParseErrorElement.appendChild(SrcTextElement)
					XliffNode.appendChild(ParseErrorElement)
				}
				ImportXliffNode := this.XfCollection.importNode(XliffNode, true)
				XfCollectionNode.appendChild(ImportXliffNode)
			}
		}
	}
	
	CreateXfDocuments()
	{
		global XfCollectionToDocumentsXslt
		this.XfDocuments := ComObjCreate("MSXML2.DOMDocument.6.0")
		this.XfDocuments.async := false
		this.XfDocuments.validateOnParse := false
		this.XfCollection.transformNodeToObject(XfCollectionToDocumentsXslt, this.XfDocuments)
	}
	
	CreateXfConsistency(TransUnitCheck := 0)
	{
		global XfCollectionToConsistencyXslt
		Xslt := ComObjCreate("MSXML2.XSLTemplate.6.0")
		Xslt.stylesheet := XfCollectionToConsistencyXslt
		XsltProc := Xslt.createProcessor()
		XsltProc.input := this.XfCollection
		XsltProc.addParameter("transUnitCheck", TransUnitCheck)
		XsltProc.transform()
		this.XfConsistency := ComObjCreate("MSXML2.DOMDocument.6.0")
		this.XfConsistency.async := false
		this.XfConsistency.validateOnParse := false
		this.XfConsistency.loadXml(XsltProc.output)
	}
	
	XliffConsider()
	{
		ErrorNode := this.XfConsistency.selectSingleNode("/xf:consistency/xf:problem[@xf:level='error']")
		if IsObject(ErrorNode) {
			if (this.HasProblem["MultipleSourceLangFiles"] 
				or (this.HasProblem["NoSourceLangFile"] and this.HasProblem["NoTargetLangFile"]))
			{
				this.XStatus.ConsiderResult := false
				return
			} else {
				this.XStatus.ConsiderResult := "E"
				return
			}
		}
		WarningNode := Consistency.selectSingleNode("/xf:consistency/xf:problem[@xf:level='warning'] | /xf:consistency/xf:invalidXliffWarning")
		if IsObject(WarningNode) {
			this.XStatus.ConsiderResult := "W"
			return
		}
		this.XStatus.ConsiderResult := true
		return
	}
	
	Consider[]
	{
		get {
			return this.XStatus.ConsiderResult
		}
	}
	
	HasProblem[Type]
	{
		get {
			ProblemNode := this.XfConsistency.selectSingleNode("/xf:consistency/xf:problem[@xf:type='" Type "']")
			if IsObject(ProblemNode) {
				return true
			} else {
				return false
			}
		}
	}
	
	ConsistencyText[]
	{
		get {
			ConText := Txt.Get("XtConsistencyText.FileSetIntro") " "
			if (this.HasProblem["MultipleSourceLangFiles"]) {
				ConText .= Txt.Get("XtConsistencyText.MultipleSourceLangFiles")
			} else if (this.HasProblem["DifferentSourceLangs"]) {
				ConText .= Txt.Get("XtConsistencyText.DifferentSourceLangs", {SourceLang: this.SourceLanguage})
			} else if (this.SourceLanguage) {
				ConText .= Txt.Get("XtConsistencyText.SourceLanguage", {SourceLang: this.SourceLanguage})
			} else {
				ConText .= Txt.Get("XtConsistencyText.NoSourceLanguage")
			}
			ConText .= " " Txt.Get("XtConsistencyText.And") " "
			if (this.HasProblem["SameTargetLangFiles"]) {
				ConText .= Txt.Get("XtConsistencyText.SameTargetLangFiles")
			} else if IsObject(this.TargetLanguageKeys) {
				for targetLang in this.TargetLanguageKeys {
					if (A_Index = 2) {
						MultipleTargetLangs := true
						break
					}
				}
				if MultipleTargetLangs {
					ConText .= Txt.Get("XtConsistencyText.TargetLanguages", {TargetLangs: this.TargetLanguageList[", "]})
				} else {
					ConText .= Txt.Get("XtConsistencyText.OneTargetLanguage", {TargetLang: this.TargetLanguageList})
				}
			} else {
				ConText .= Txt.Get("XtConsistencyText.NoTargetLanguage")
			}
			return ConText
		}
	}
	
	CreateXml()
	{
		global XfCollectionToXliffXslt
		this.Xml := ComObjCreate("MSXML2.DOMDocument.6.0")
		this.Xml.async := false
		this.Xml.validateOnParse := false
		this.XfCollection.transformNodeToObject(XfCollectionToXliffXslt, this.Xml)
	}
	
	TargetLanguageList[Seperator := "|"]
	{
		get {
			for targetLang in this.TargetLanguageKeys {
				if (A_Index > 1) {
					TargetLangList .= Seperator
				}
				TargetLangList .= targetLang
			}
			return TargetLangList
		}
	}
	
	ConsistencyReport()
	{
		ReportsFolder := Config.Folder["ConsistencyReports"]
		ReportFile := this.XStatus.ConsistencyReport
		if not (ReportFile and FileExist(ReportsFolder "\" ReportFile)) {
			this.CreateConsistencyReport()
		}
		Run, % ReportsFolder "\" this.XStatus.ConsistencyReport
	}
	
	CreateConsistencyReport()
	{
		global XfConsistencyXliffDoc, XfConsistencyReportSourceToReportXslt, XfConsistencyReportLanguageCode
		
		XfConsistencyReportSource := ComObjCreate("MSXML2.DOMDocument.6.0")
		XfConsistencyReportSource.async := false
		XfConsistencyReportSource.validateOnParse := false
		XfConsistencyReportSourceRoot = <?xml version="1.0"?>`n<xf:consistencyReportSource xmlns="urn:oasis:names:tc:xliff:document:1.2" xmlns:xf="xl:ffer"></xf:consistencyReportSource>
		XfConsistencyReportSource.loadXml(XfConsistencyReportSourceRoot)
		XfConsistencyReportSourceNode := XfConsistencyReportSource.documentElement
		ImportXfConsistencyXliffDoc := XfConsistencyReportSource.importNode(XfConsistencyXliffDoc.documentElement, true)
		XfConsistencyReportSourceNode.appendChild(ImportXfConsistencyXliffDoc)
		if this.XStatus.ConsiderResult {
			this.CreateXfConsistency(1)
		}
		ImportXfConsistency := XfConsistencyReportSource.importNode(this.XfConsistency.documentElement, true)
		XfConsistencyReportSourceNode.appendChild(ImportXfConsistency)
		ImportXfDocuments := XfConsistencyReportSource.importNode(this.XfDocuments.documentElement, true)
		XfConsistencyReportSourceNode.appendChild(ImportXfDocuments)
		LocalSemanticUiCss := Config.Folder["SemanticUiCssFolder"] "\semantic.min.css"
		if (FileExist(LocalSemanticUiCss)) {
			LocalSemanticUiCss := StrReplace(LocalSemanticUiCss, "\", "/")
			VarSetCapacity(Var, StrPut(LocalSemanticUiCss, "UTF-8"), 0), StrPut(LocalSemanticUiCss, &Var, "UTF-8")
			While Code := NumGet(Var, A_Index - 1, "UChar")
				Res .= (Chr:=Chr(Code)) ~= "[0-9A-Za-z:/;?@,&=+$#.]" ? Chr : Format("%{:02X}", Code)
			LocalCssUrl := "file:///" Res
		}
		Xslt := ComObjCreate("MSXML2.XSLTemplate.6.0")
		Xslt.stylesheet := XfConsistencyReportSourceToReportXslt
		XsltProc := Xslt.createProcessor()
		XsltProc.input := XfConsistencyReportSource
		XsltProc.addParameter("language", XfConsistencyReportLanguageCode)
		FormatTime, CurrentDateTime,, yyyy-MM-dd HH:mm
		FormatTime, CurrentDateTimeFileName,, yyyy-MM-dd HHmmss
		XsltProc.addParameter("currentDateTime", CurrentDateTime)
		if LocalCssUrl {
			XsltProc.addParameter("cssUrl", LocalCssUrl)
		}
		XsltProc.transform()
		ReportFileName := "consistency-report " CurrentDateTimeFileName ".html"
		ReportFile := FileOpen(Config.Folder["ConsistencyReports"] "\" ReportFileName, "w", "UTF-8-RAW")
		if !IsObject(ReportFile)
		{
			MsgBox, 8208, % Txt.Get("WriteXmlResultFileErrorMsgBoxTitle")
				, % Txt.Get("WriteXmlResultFileErrorMsgBoxText", {File: ReportFileName})
			return
		}
		ReportFile.Write(XsltProc.output)
		ReportFile.Close()
		this.XStatus.ConsistencyReport := ReportFileName
	}
	
	List[Mode := "Object"]
	{
		get {
			if (Mode = "ParameterString") {
				;* XSLT parameter as String e.g."de|frenchfr|en"
				for key in this.XList {
					if this.XList[key].HasKey("Check") {
						if (MoreThanOne) {
							String .= "|"
						}
						String .= (this.XList[key].HasKey("DataSetName"))
							? this.XList[key].DataSetName "" key
							: key
						MoreThanOne := 1
					}
				}
				return String
			} else if (Mode = "IniString") {
				;* Save in Ini as String e.g. "1de|1frenchfr|0en"
				for key in this.XList {
					if MoreThanOne {
						String .= "|"
					}
					String .= (this.XList[key].HasKey("Check")) ? "1" : "0"
					String .= (this.XList[key].HasKey("DataSetName"))
						? this.XList[key].DataSetName "" key
						: key
					MoreThanOne := 1
				}
				return String
			} else if (Mode = "SelectionKeys") {
				SelectionKeys := {}
				for key in this.XList {
					if this.XList[key].HasKey("Check") {
						SelectionKeys[key] := 1
					}
				}
				return SelectionKeys
			} else {
				return this.XList
			}
		}
		set {
			if (Mode = "Merge") {
				for key, item in value {
					if this.XList.HasKey(key) {
						this.XList[key] := item
					}
				}
			} else if (Mode = "MergeIniString") {
				;* Load from Ini as String e.g. "1de|1franzsischfr|0en"
				Loop, Parse, value, |, %A_Space%%A_Tab%
				{
					Checked := SubStr(A_LoopField, 1, 1)
					KeyString := SubStr(A_LoopField, 3)
					if (CharFoundPos := InStr(KeyString, "")) {
						Key :=  SubStr(KeyString, CharFoundPos + 1)
						DataSetName := SubStr(KeyString, 1, CharFoundPos - 1)
					} else {
						Key := KeyString
					}
					if this.XList.HasKey(Key) {
						this.XList[Key] := {}
						if Checked {
							this.XList[Key, "Check"] := 1
						}
						if CharFoundPos {
							this.XList[Key, "DataSetName"] := DataSetName
						}
					}
				}
			}
		}
	}
}