<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xlf="urn:oasis:names:tc:xliff:document:1.2"
	xmlns:xf="xl:ffer"
	xmlns="http://www.w3.org/1999/xhtml"
	exclude-result-prefixes="xlf xf">

	<xsl:output method="xml"
		omit-xml-declaration="yes" />

	<xsl:param name="language"/>

	<xsl:param name="currentDateTime"/>

	<xsl:param name="cssUrl" select="false()"/><!-- can be used to make CSS work offline -->

	<xsl:variable name="sourceOrTarget">
		<xsl:choose>
			<xsl:when test="/xf:consistencyReportSource/xlf:xliff/xlf:file/@source-language = $language">source</xsl:when>
			<xsl:otherwise>target</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

	<xsl:variable name="css">
		<xsl:choose>
			<xsl:when test="$cssUrl">
				<xsl:value-of select="$cssUrl"/>
			</xsl:when>
			<xsl:otherwise>https://cdn.jsdelivr.net/npm/semantic-ui/dist/semantic.min.css</xsl:otherwise>
		</xsl:choose>
	</xsl:variable><!-- https://cdn.jsdelivr.net/npm/semantic-ui@2.2/dist/semantic.min.css -->

	<xsl:key name="problem-by-type" match="xf:problem" use="@xf:type"/>

	<xsl:key name="problem-by-document" match="xf:problem" use="@xf:document"/>

	<xsl:template match="/">
		<xsl:text disable-output-escaping="yes">&lt;!DOCTYPE html&gt;&#xA;</xsl:text>
		<html>
			<head>
				<meta charset="UTF-8" />
				<meta http-equiv="X-UA-Compatible" content="IE=edge"/>
				<meta name="viewport" content="width=device-width, initial-scale=1"/>
				<title>
					<xsl:call-template name="xliffText">
						<xsl:with-param name="transUnitId">ReportTitle</xsl:with-param>
					</xsl:call-template>
				</title>
				<link href="{$css}" rel="stylesheet" type="text/css"/>
				<style type="text/css">
					pre {
						white-space: pre-wrap;
						-moz-tab-size: 2;
						-o-tab-size: 2;
						tab-size: 2;
					}
				</style>
			</head>
			<body>
				<div class="ui sizer vertical segment">
					<div class="ui text container">
						<h1 class="ui header">
							<xsl:call-template name="xliffText">
								<xsl:with-param name="transUnitId">ReportHeader</xsl:with-param>
							</xsl:call-template>
							<div class="sub header">
								<xsl:text>XLIFFer </xsl:text>
								<xsl:value-of select="$currentDateTime"/>
							</div>
						</h1>
						<div class="ui small top attached header">
							<xsl:call-template name="xliffText">
								<xsl:with-param name="transUnitId">ReportDocumentsHeader</xsl:with-param>
							</xsl:call-template>
						</div>
						<div class="ui attached segment">
							<p>
								<xsl:call-template name="xliffText">
									<xsl:with-param name="transUnitId">ReportDocumentsFolderIntro</xsl:with-param>
								</xsl:call-template>
								<xsl:text> </xsl:text>
								<xsl:value-of select="xf:consistencyReportSource/xf:documents/@xf:folder"/>
							</p>
							<div class="ui list">
								<xsl:for-each select="xf:consistencyReportSource/xf:documents/xf:document">
									<div class="item">
										<i class="file outline icon"></i>
										<div class="content">
											<xsl:value-of select="."/>
										</div>
									</div>
								</xsl:for-each>
							</div>
						</div>
						<xsl:apply-templates select="xf:consistencyReportSource/xf:consistency"/>
					</div>
				</div>
			</body>
		</html>
	</xsl:template>

	<xsl:template match="xf:consistency">
		<xsl:choose>
			<xsl:when test="xf:*">
				<xsl:call-template name="overall-source-language-problems"/>
				<xsl:call-template name="overall-target-language-problems"/>
				<xsl:call-template name="overall-original-problems"/>
				<xsl:call-template name="overall-datatype-problems"/>
				<xsl:apply-templates select="xf:problem[@xf:document] | xf:invalidXliffWarning"/>
			</xsl:when>
			<xsl:otherwise>
				<div class="ui success message" data-message-type="ok">
					<div class="header">
						<xsl:call-template name="xliffText">
							<xsl:with-param name="transUnitId">ConsistentHeader</xsl:with-param>
						</xsl:call-template>
					</div>
					<p>
						<xsl:call-template name="xliffText">
							<xsl:with-param name="transUnitId">ConsistentText</xsl:with-param>
						</xsl:call-template>
					</p>
				</div>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="overall-source-language-problems">
		<xsl:if test="xf:problem/@xf:group = 'overall-source-language'">
			<h2 class="ui small header">
				<xsl:call-template name="xliffText">
					<xsl:with-param name="transUnitId">OverallSourceLanguageProblemsHeader</xsl:with-param>
				</xsl:call-template>
			</h2>
			<xsl:apply-templates select="xf:problem[@xf:group = 'overall-source-language']" mode="group"/>
		</xsl:if>
	</xsl:template>

	<xsl:template name="overall-target-language-problems">
		<xsl:if test="xf:problem/@xf:group = 'overall-target-language'">
			<h2 class="ui small header">
				<xsl:call-template name="xliffText">
					<xsl:with-param name="transUnitId">OverallTargetLanguageProblemsHeader</xsl:with-param>
				</xsl:call-template>
			</h2>
			<xsl:apply-templates select="xf:problem[@xf:group = 'overall-target-language']" mode="group"/>
		</xsl:if>
	</xsl:template>

	<xsl:template name="overall-original-problems">
		<xsl:if test="xf:problem/@xf:group = 'overall-original'">
			<h2 class="ui small header">
				<xsl:call-template name="xliffText">
					<xsl:with-param name="transUnitId">OverallOriginalProblemsHeader</xsl:with-param>
				</xsl:call-template>
			</h2>
			<xsl:apply-templates select="xf:problem[@xf:group = 'overall-original']" mode="group"/>
		</xsl:if>
	</xsl:template>

	<xsl:template name="overall-datatype-problems">
		<xsl:if test="xf:problem/@xf:group = 'overall-datatype'">
			<h2 class="ui small header">
				<xsl:call-template name="xliffText">
					<xsl:with-param name="transUnitId">OverallDatatypeProblemsHeader</xsl:with-param>
				</xsl:call-template>
			</h2>
			<xsl:apply-templates select="xf:problem[@xf:group = 'overall-datatype']" mode="group"/>
		</xsl:if>
	</xsl:template>

	<xsl:template match="xf:problem[@xf:document]">
		<xsl:if test="count(. | key('problem-by-document', @xf:document)[1]) = 1">
			<xsl:call-template name="documentProblemsHeader">
				<xsl:with-param name="document" select="@xf:document"/>
			</xsl:call-template>
		</xsl:if>
		<xsl:apply-templates select="." mode="message"/>
	</xsl:template>

	<xsl:template name="documentProblemsHeader">
		<xsl:param name="document"/>
		<h2 class="ui small header">
			<xsl:call-template name="xliffText">
				<xsl:with-param name="transUnitId">DocumentProblemsHeader</xsl:with-param>
			</xsl:call-template>
			<xsl:text> </xsl:text>
			<xsl:value-of select="$document"/>
		</h2>
	</xsl:template>

	<xsl:template match="xf:problem" mode="group">
		<xsl:if test="count(. | key('problem-by-type', @xf:type)[1]) = 1">
			<xsl:apply-templates select="." mode="message">
				<xsl:with-param name="group" select="true()"/>
			</xsl:apply-templates>
		</xsl:if>
	</xsl:template>

	<xsl:template match="xf:problem" mode="message">
		<xsl:param name="group" select="false()"/>
		<div class="ui {@xf:level} message" data-message-type="{@xf:type}">
			<div class="header">
				<xsl:call-template name="xliffText">
					<xsl:with-param name="transUnitId">
						<xsl:value-of select="@xf:type"/>
						<xsl:text>Header</xsl:text>
					</xsl:with-param>
					<xsl:with-param name="document" select="@xf:document"/>
				</xsl:call-template>
			</div>
			<p>
				<xsl:call-template name="xliffText">
					<xsl:with-param name="transUnitId">
						<xsl:value-of select="@xf:type"/>
						<xsl:text>Text</xsl:text>
					</xsl:with-param>
					<xsl:with-param name="document" select="@xf:document"/>
				</xsl:call-template>
			</p>
			<xsl:if test="@xf:type != 'NoSourceLangFile' and @xf:type != 'NoTargetLangFile'">
				<div class="ui tiny header">
					<xsl:call-template name="xliffText">
						<xsl:with-param name="transUnitId">PointerHeader</xsl:with-param>
					</xsl:call-template>
				</div>
				<div class="ui list">
					<xsl:choose>
						<xsl:when test="$group">
							<xsl:apply-templates select="key('problem-by-type', @xf:type)/xf:pointer"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates select="xf:pointer"/>
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</xsl:if>
		</div>
	</xsl:template>

	<xsl:template match="xf:pointer">
		<div class="item">
			<i class="caret right icon"></i>
			<div class="content">
				<xsl:value-of select="."/>
			</div>
		</div>
	</xsl:template>

	<xsl:template match="xf:invalidXliffWarning">
		<xsl:variable name="document" select="@xf:document"/>
		<xsl:if test="not(../xf:problem[@xf:document = $document])">
			<xsl:call-template name="documentProblemsHeader">
				<xsl:with-param name="document" select="$document"/>
			</xsl:call-template>
		</xsl:if>
		<xsl:variable name="parseErrorLinePos">
			<xsl:call-template name="xliffText">
				<xsl:with-param name="transUnitId">InvalidXliffParseErrorLineposIntro</xsl:with-param>
			</xsl:call-template>
			<xsl:text> </xsl:text>
			<xsl:value-of select="xf:linepos"/>
		</xsl:variable>
		<div class="ui warning message" data-message-type="InvalidXliff" data-error-code="{xf:errorCode}">
			<div class="header">
				<xsl:call-template name="xliffText">
					<xsl:with-param name="transUnitId">InvalidXliffWarningHeader</xsl:with-param>
					<xsl:with-param name="document" select="$document"/>
				</xsl:call-template>
			</div>
			<p>
				<xsl:call-template name="xliffText">
					<xsl:with-param name="transUnitId">InvalidXliffWarningText</xsl:with-param>
					<xsl:with-param name="document" select="$document"/>
				</xsl:call-template>
			</p>
			<div class="ui tiny header" title="{$parseErrorLinePos}">
				<xsl:call-template name="xliffText">
					<xsl:with-param name="transUnitId">InvalidXliffParseErrorLineIntro</xsl:with-param>
				</xsl:call-template>
				<xsl:text> </xsl:text>
				<xsl:value-of select="xf:line"/>:
			</div>
			<pre>
				<code class="xml">
					<xsl:value-of select="xf:srcText"/>
				</code>
			</pre>
			<p>
				<xsl:value-of select="xf:reason"/>
			</p>
		</div>
	</xsl:template>

	<xsl:template name="xliffText">
		<xsl:param name="transUnitId"/>
		<xsl:param name="document" select="false()"/>
		<xsl:choose>
			<xsl:when test="$sourceOrTarget = 'source'">
				<xsl:apply-templates select="/xf:consistencyReportSource/xlf:xliff/xlf:file[not(@target-language) and @source-language = $language]/xlf:body/xlf:trans-unit[@id = $transUnitId]/xlf:source/node()">
					<xsl:with-param name="document" select="$document"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="/xf:consistencyReportSource/xlf:xliff/xlf:file[@target-language = $language]/xlf:body/xlf:trans-unit[@id = $transUnitId]/xlf:target/node()">
					<xsl:with-param name="document" select="$document"/>
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xlf:x[@ctype = 'x-document']">
		<xsl:param name="document"/>
		<xsl:text>‹</xsl:text>
		<xsl:value-of select="$document"/>
		<xsl:text>›</xsl:text>
	</xsl:template>

	<xsl:template match="xlf:x[@ctype = 'x-linebreak']">
		<xsl:param name="document" select="false()"/>
		<br/>
	</xsl:template>

	<xsl:template match="xlf:x[@ctype = 'x-space']">
		<xsl:param name="document" select="false()"/>
		<xsl:text>&#x20;</xsl:text>
	</xsl:template>

	<xsl:template match="xlf:x[@ctype = 'x-nbsp']">
		<xsl:param name="document" select="false()"/>
		<xsl:text>&#160;</xsl:text>
	</xsl:template>

	<xsl:template match="text()">
		<xsl:param name="document" select="false()"/>
		<xsl:value-of select="normalize-space()"/>
	</xsl:template>

</xsl:stylesheet>